@@ -12,10 +12,10 @@ Matrix::column(const mdsize c) const {
   MatrixBuffer* p = (MatrixBuffer*)buffer;
   if(p->symmflag) panic("Symmetric matrix.", __FILE__, __LINE__);
   if(c >= p->ncols) return vector<mdreal>();
-  vector<mdreal> values(p->nrows, p->rlnan);
+  vector<mdreal> output(p->nrows, p->rlnan);
   unordered_map<mdsize, Array>& rowdata = p->rowdata;
   for(unordered_map<mdsize, Array>::iterator it = rowdata.begin();
       it != rowdata.end(); it++)
-    values[it->first] = it->second[c];
-  return values;
+    output[it->first] = it->second[c];
+  return output;
 }
