@@ -3,11 +3,14 @@
 
 #include "nro.h"
 
+#define nro_NMAX_FALSE 20
+
 /*
- *
+ * Synchronous permutations of multiple dimensions in parallel.
+ * Only data points with no unusable values are included.
  */
 static string
-nro_permute_sync(abacus::Matrix& stats, vector<vector<mdreal> >& vectors,
+nro_permute_sync(abacus::Matrix& stats, vector<vector<mdreal> >& points,
 		 const vector<mdsize>& bmus, const Topology& topo,
 		 const mdsize ncycl) {
   stats = abacus::Matrix();
@@ -15,19 +18,19 @@ nro_permute_sync(abacus::Matrix& stats, vector<vector<mdreal> >& vectors,
   /* Create a simulation engine. R-style indexing switched to C++. */
   Engine eng(topo);
   mdsize ndata = 0;
-  for(mdsize i = 0; i < vectors.size(); i++) {
-    string e = eng.insert(long2string(i), (bmus[i] - 1), vectors[i]);
-    ndata += (e.size() < 1);
-    vectors[i].clear(); /* reduce memory footprint */
+  for(mdsize i = 0; i < points.size(); i++) {
+    string e = eng.insert(long2string(i), (bmus[i] - 1), points[i]);
+    ndata += (e.size() < 1); /* excluded if any missing value */
+    points[i].clear(); /* reduce memory footprint */
   }
 
   /* Observed component planes. */
   vector<vector<mdreal> > observed = eng.average();
   mdsize nvars = observed.size();
   if(nvars < 1) return "Permutation failed.";
-  
+
   /* Observed statistics. */
-  vector<mdreal> scores(nvars);
+  vector<mdreal> scores(nvars, 0.0);
   for(mdsize j = 0; j < nvars; j++)
     scores[j] = abacus::statistic(observed[j], "sd");
   
@@ -35,6 +38,7 @@ nro_permute_sync(abacus::Matrix& stats, vector<vector<mdreal> >& vectors,
   mdsize nfalse = 0;
   mdsize ntotal = 0;
   vector<abacus::Empirical> fnull(nvars);
+  
   for(mdsize n = 0; n < ncycl; n++) {
     if(nfalse/nvars >= nro_NMAX_FALSE) break;
     
@@ -46,7 +50,7 @@ nro_permute_sync(abacus::Matrix& stats, vector<vector<mdreal> >& vectors,
     if(permuted.size() < 1) return "Permutation failed.";
     
     /* Permuted statistics. */
-    vector<mdreal> nulls(nvars);
+    vector<mdreal> nulls(nvars, 0.0);
     for(mdsize j = 0; j < nvars; j++)
       nulls[j] = abacus::statistic(permuted[j], "sd");
     
@@ -59,7 +63,7 @@ nro_permute_sync(abacus::Matrix& stats, vector<vector<mdreal> >& vectors,
       fnull[j].add(nulls[j], 1.0);
     ntotal++;
   }
-  
+
   /* Estimate statistics. */
   for(mdsize j = 0; j < nvars; j++) {
     stats.insert(0, j, scores[j]);
@@ -75,8 +79,10 @@ nro_permute_sync(abacus::Matrix& stats, vector<vector<mdreal> >& vectors,
  *
  */
 RcppExport SEXP
-nro_permute(SEXP topo_R, SEXP bmus_R, SEXP data_R, SEXP numcycl_R) {
+nro_permute(SEXP topo_R, SEXP bmus_R, SEXP data_R,
+	    SEXP numcycl_R, SEXP lag_R) {
   vector<mdsize> numcycl = nro::vector2sizes(numcycl_R);
+  mdreal lag = as<mdreal>(lag_R);
   
   /* Check inputs. */
   vector<mdsize> bmus = nro::vector2sizes(bmus_R);
@@ -92,9 +98,12 @@ nro_permute(SEXP topo_R, SEXP bmus_R, SEXP data_R, SEXP numcycl_R) {
   punos::Topology topo = reals2topology(topodata);
   if(topo.size() < 1) return CharacterVector("Unusable topology.");
 
-  /* Asynchronous permutations. */
+  /* Asynchronous permutations. That is, separate synchronous
+     permutations for each single dimension. */
   abacus::Matrix stats;
-  for(mdsize j = 0; j < vectors[0].size(); j++) {
+  time_t stamp = time(NULL);
+  mdsize nvars = vectors[0].size();
+  for(mdsize j = 0; j < nvars; j++) {
     
     /* Extract data values. */
     vector<vector<mdreal> > column(vectors.size());
@@ -109,6 +118,15 @@ nro_permute(SEXP topo_R, SEXP bmus_R, SEXP data_R, SEXP numcycl_R) {
     /* Update results. */
     for(mdsize i = 0; i < batch.size(); i++)
       stats.insert(i, j, batch.value(i, 0));
+    
+    /* Progress message. */
+    if(lag < 0.0) continue;
+    mdreal dt = difftime(time(NULL), stamp); 
+    if(dt >= lag) {
+      string msg = ("completed in " + medusa::time2text(dt));
+      Rprintf("%d / %d %s\n", (j + 1), nvars, msg.c_str());
+      stamp = time(NULL);
+    }
   }
     
   /* Return results. */
